function value = computeValue(varargin)
% COMPUTEVALUE calculates the value of a pixel in k-space for a phantom
%
% VALUE=COMPUTEVALUE(POS,SIZE,...) calculates the value of relative 2d
%                                  pixel POS in k-space (-0.5,...,0.5) for a
%                                  Shepp-Logan-Phantom of size SIZE and
%                                  saves the reulst in VALUE
% 
% VALUE=COMPUTEVALUE(...,FINE,...) decides the level of detail (true=detailed)
%
% VALUE=COMPUTEVALUE(...,MATRIX,...) creates the value for the Phantom
%                                    specified in MATRIX, where MATRIX must
%                                    have following form:
%                                    -1st column: length of halfaxis in x-direction
%                                    -2nd column: length of halfaxis in y-direction
%                                    -3rd column: shift in x-direction
%                                    -4th column: shift in y-direction
%                                    Above values should be relativ (-1,...,1)
%                                    -5th column: rotation angle in degree
%                                    -6th column: densitivity level
%                                    -7th column: 1 if it should only shown if
%                                                 FINE=true, 0 else
%
% VALUE=COMPUTEVALUE(...,OFFSET,...) specifies the offset of the phantom in
%                                    the image. Offset should be a n x 3 x m
%                                    matrix, where n is the number of different
%                                    points in time of the shift.
%                                    If m=1 the whole phantom will be shifted
%                                    If m=Number of Objects in the phantom,
%                                    each Object will be shifted seperatly.
%                                    The three columns include the information:
%                                        -shift in x-direction in pixel
%                                        -shift in y-direction in pixel
%                                        -rotationangle in degree
%
% VALUE=COMPUTEVALUE(...,METHOD,...) defines the method to transform the
%                                    phantom from k-space to image space.
%                                    Possible options: 'fft','ifft'
%
% VALUE=COMPUTEVALUE(...,TIME,...) defines the point in time of the
%                                  shift, only useful if OFFSET is set
%
% Author: Johannes Churt

%Sanity checks
error(nargoutchk(0, 1, nargout));

[pos,M,offset,time,fine,cv,method]=parseInput(varargin{:});
if (strcmp(method,'ifft'))
    pos=-pos;
end

%Delegate the calculating
value=cv(pos,M,offset,time,fine);
end

function [pos,M,offset,time,fine,cv,method]=parseInput(varargin)
% PARSEINPUT parses the input for computeValue
% Author: Johannes Churt

%Initalization values
pos=[];
M=[];
s=0;
offset=[];
time=1;
method='fft';
fine=false;
cv=@computeValueNone;
setsize=false;
settime=false;

for k=1:nargin
    if (islogical(varargin{k}))
        %Logicals are for the level of detail
        fine=varargin{k};
    elseif (ischar(varargin{k}))
        %String reprasents the method of transforming to image-space
        switch varargin{k}
            case 'fft'
                method='fft';
            case 'ifft'
                method='ifft';
            otherwise
                error('SheppLogan:computeValue:invalidInputArgs',...
                   'Unknown transformation method %s.',varargin{k});
        end
    elseif (numel(varargin{k})==1)
        %First scalar is the size, second the time
        if (setsize && settime)
            error('SheppLogan:computeValue:invalidInputArgs','Invalid input arguments.\n');
        elseif (setsize)
            time=varargin{k};
            settime=true;
        else
            s=varargin{k};
            setsize=true;
        end
    elseif (ndims(varargin{k})==2 && (size(varargin{k},1)==2))
        % Position in k-space
        pos=varargin{k};
        pos(2,:)=-pos(2,:);
    elseif (ndims(varargin{k})==2 && size(varargin{k},2)==7)
        %User specified phantom
        M=varargin{k};
    elseif (ndims(varargin{k})==2 && size(varargin{k},2)==3)
        %An offset matrix moving the whole phantom
        offset=varargin{k};
        cv=@computeValueAll;
    elseif (ndims(varargin{k})==3 && size(varargin{k},2)==3)
        %An offset matrix moving components speratly
        offset=varargin{k};
        cv=@computeValueSpecific;
    else
        error('SheppLogan:computeValue:invalidInputArgs','Invalid input arguments.\n');
    end
end

if (s==0)
    %We are lacking an image size
    error('SheppLogan:computeValue:invalidInputArgs','No image size.\n');
end
if (isempty(pos))
    %We are lacking a position
    error('SheppLogan:computeValue:invalidInputArgs','No position specified.\n');
end

if (isempty(M))
    %No phantom specified
    M=sheppLogan;
end

if (~isempty(offset))
    %We have a offset matrix, so scale it
    offset=[offset(:,1:2,:)/s*2 offset(:,3,:)*pi/180];
    if (time>size(offset,1))
        string=['th' 'st' 'nd' 'rd' repmat('th',1,6)];
        error('SheppLogan:computeValue:invalidInputArgs',...
            ['Offset matrix has only %d entries in time-direction, but '...
            '%d%s point of time was requested.\n'],...
            size(offset,1),time,string(2*rem(time,10)+1:2*rem(time,10)+2));
    end
end

numberellipses=size(M,1)-sum(M(:,7))*(fine==false);
if ((ndims(offset)==3) && (size(offset,3)~=numberellipses))
    error('SheppLogan:computeValue:invalidInputArgs',...
        'Offset matrix must have %d entries in z-direction.',numberellipses);
end
pos=rotation(pos,pi)*s;
end
