% Authors: 
% Robert Grimm robert.grimm@cs.fau.de [1]
% Andreas Fieselmann [1]
% [1] Pattern Recognition Lab, FAU Erlangen-Nuremberg, Erlangen, Germany
% Last change:  09.05.2012

function [ aif, tacs, tvec ] = generateCurves( aifModel, tissues, tMax, t0 )
%GENERATECURVES generate tissue attenuation curves according to Tofts & Kermode model

    if nargin==2
        tMax = 300;
    end
    if nargin<=3
        t0 = 0;
    end

    dt = 0.1;                   % unit: s

    tvec = 0 : dt : tMax;       % unit: s, time vector for all three time-attenuation curves

    aif = [];
    tacs = [];

    if ischar(aifModel)
        if strcmp(aifModel, 'Parker')==true
            aif = Parker(tvec, t0, 1);
        elseif strcmp(aifModel, 'Weinmann')==true
            aif = Weinmann(tvec, t0);
        else
            error('dynaSL:generateCurves:unknownAIF', 'Error: unknown aif model %s \n', aifModel);
        end
    else
        aif = aifModel; % user-defined AIF
        % TODO: re-sample (interpolate) to steps of dt!
    end
    
    tacs = GetTissueTacTofts(aif,tvec, tissues.Ktrans, tissues.kep, tissues.ve, tissues.vp);
   
end


% compute tissue time attenuation curves according to Tofts model
function tacs = GetTissueTacTofts(aif,t, Ktrans, kep, ve, vp)

    % Note: This approach assumes dt to be sufficiently low (~0.1 s) such that
    % discretization before convolution yields approx. same results as
    % convolution before discretization.

    % TODO auto-completion if one of Ktrans, kep, ve is missing
    % only Ktrans and kep are used anyways!
    
    nCurves = max([numel(Ktrans), numel(ve), numel(kep), numel(vp)]);

    if numel(ve)~=nCurves || numel(kep)~=nCurves || numel(vp)~=nCurves || numel(Ktrans)~=nCurves
      error('dynaSL:generateCurves:GetTissueTacTofts:invalidParams','Number of elements in tissue Ktrans, ve, kep and vp do not match\n');
    end

    for i=1:nCurves
        rI = exp(-kep(i) * t );
        tacI = Ktrans(i) * conv(aif, rI);
        tacI = tacI(1:numel(aif));
        tacs(i,:) = vp(i) * aif + tacI;
    end

end


function Cp = Parker( t, t0, ts )

    t2 = (t-t0)/ts;

    A = [0.809, 0.33]*60;   % unit: mmol.min --> mmol.s
    T = [0.17046, 0.365]*60; % unit: min --> s
    sigma = [0.0563, 0.132]*60;  % unit: min --> s
    alpha = 1.050;
    beta = 0.1685/60;  % unit: 1/min --> 1/s
    tau = 0.483*60;         % unit: min --> s
    s = 38.078/60;         % unit: 1/min --> 1/s

    Cb = A(1) / (sigma(1)*sqrt(2*pi))*exp(-(t2-T(1)).^2/(2*sigma(1)^2)) + ...
        A(2) / (sigma(2)*sqrt(2*pi))*exp(-(t2-T(2)).^2/(2*sigma(2)^2)) + ...
        alpha * exp(-beta*t2) ./ (1 + exp(-s*(t2-tau)));

    % we want the plasma concentration
    Hct = 0.42;

    Cp = Cb / (1-Hct);

end


function Cp = Weinmann( t, t0 )

    t2 = max(t-t0, 0);
    D = 0.1; % dose: mmol/l
    a = [3.99, 4.78]; % kg/l
    m = [0.144, 0.0111]/60; % 1/min --> 1/s
    Cp = D * (a(1) * exp(-m(1)*t2) + a(2) * exp(-m(2)*t2));

end

