% Author: Robert Grimm robert.grimm@cs.fau.de
% Pattern Recognition Lab, FAU Erlangen-Nuremberg, Erlangen, Germany
% Last change:  09.05.2012

function [ samples, PEorder ] = sampleCartesian( baseres, perfusionT0, perfusionTime, samplingT0, samplingTend, reordering, aifModel )
%SAMPLECARTESIAN Example for Cartesian trajectories
%   baseres: resolution
%   perfusionT0: start of contrast agent injection
%   perfusionTime: time of simulated perfusion process
%   samplingT0-samplingTend: sampling time
%   reordering: 'linear', 'centric', or 'inverse-centric'
%   aifModel: 'Parker' or 'Weinmann', or user-defined aif

    % compute k-space grid
    [kx,ky] = meshgrid((-baseres/2):(baseres/2-1),(-baseres/2):(baseres/2-1));
    kx = kx / baseres;
    ky = ky / baseres;

    % compute order of PE lines
    if ischar(reordering)
        if strcmp(reordering, 'linear')==true
             PEorder = 1:baseres;
        elseif strcmp(reordering, 'centric')==true
            PEorder(1,:) = (baseres/2+1):baseres;
            PEorder(2,:) = (baseres/2):-1:1;
            PEorder = PEorder(:);   
        elseif strcmp(reordering, 'inverse-centric')==true
            PEorder(1,:) = 1:(baseres/2);
            PEorder(2,:) = baseres:-1:(baseres/2+1);
            PEorder = PEorder(:);
        else
            error('dynaSL:sampleCartesian:unknownReordering', 'Unknown reordering ''%s''\n', reordering);
        end
    else
        if numel(reordering)==baseres
            PEorder=reordering;
        else
            error('dynaSL:sampleCartesian:invalidReordering', 'Number of elements in reordering must match baseresolution.');
        end
    end

    % group readouts: one readout is assumed to be instantaneous
    numReadouts = numel(PEorder);

    for i=1:numReadouts
      kxi = kx(PEorder(i), :);
      kyi = ky(PEorder(i), :);
      kPos{i}(1,:) = kxi(:);
      kPos{i}(2,:) = kyi(:);
    end

    % generate curves and convert to signal intensities
    tissues = tissuePresets(0);
    [aif, tacs, tvec] = generateCurves(aifModel, tissues, perfusionTime, perfusionT0);

    % compute readout times (discretized at uniform intervals of TR)
    tReadout = linspace(samplingT0, samplingTend, numReadouts);
    samplingT = cell([1, numReadouts]);
    tvecIdx = 1;
    for i=1:numReadouts
        while tvec(tvecIdx) < tReadout(i)
            tvecIdx = tvecIdx + 1;
        end
        samplingT{i} = tvecIdx;  % corresponding _indices_ into tvec / aif and tacs
    end


    % evaluate phantom
    samples = getDynaSamples(baseres, kPos, samplingT, aif, tacs);
    
    

end

