function N=sheppLogan2D(varargin)
% SHEPPLOGAN2D creates a Shepp-Logan phantom in k-space
%
% N=SHEPPLOGAN2D creates a standard 256 pixel phantom in k-space and saves
%                the result in N
%
% N=SHEPPLOGAN2D(SIZE,...) creates a standard SIZE pixel phantom in k-space
% 
% N=SHEPPLOGAN2D(...,FINE,...) specifies as a logical the detail of level
%                             (true=detailed)
%
% N=SHEPPLOGAN2D(...,MATRIX,...) creates the phantom specified in MATRIX,
%                                where MATRIX must have following form:
%                                -1st column: length of halfaxis in x-direction
%                                -2nd column: length of halfaxis in y-direction
%                                -3rd column: shift in x-direction
%                                -4th column: shift in y-direction
%                                Above values should be relativ (-1,...,1)
%                                -5th column: rotation angle in degree
%                                -6th column: densitivity level
%                                -7th column: 1 if it should only shown if
%                                             FINE=true, 0 else
%
% N=SHEPPLOGAN2D(...,OFFSET,...) specifies the offset of the phantom in the
%                                image. Offset should be a n x 3 x m
%                                matrix, where n is the number of different
%                                points in time of the shift.
%                                If m=1 the whole phantom will be shifted
%                                If m=Number of Objects in the phantom,
%                                each Object will be shifted seperatly.
%                                The three columns include the information:
%                                   -shift in x-direction in pixel
%                                   -shift in y-direction in pixel
%                                   -rotationangle in degree
%
% N=SHEPPLOGAN2D(...,METHOD,...) defines the method to transform the
%                                phantom from k-space to image space.
%                                Possible options: 'fft','ifft'
%
% N=SHEPPLOGAN2D(...,TIME,...) defines the point in time of the shift,
%                              only useful if OFFSET is set
%
% Author: Johannes Churt

%Sanity check
error(nargoutchk(0, 1, nargout));

%Initalization
[M,size,offset,time,fine,cv,method]=parseInput(varargin{:});
L=floor(size/2);
k=-L:L-1;
K=2*L;
N=zeros(K);
if strcmp(method,'ifft')
    k=-k;
end

%Calculate the phantom
for n=1:K
	kx=repmat(k(n),[1,K]);
	N(:,n)=cv([-kx;k],M,offset,time,fine);
end

%Transform the matrix to standardform
N=N/max(abs(N(:)));

%Uncomment to view result in image space
if (strcmp(method,'fft'))
    X=abs(fftshift(fft2(ifftshift(N))));
elseif (strcmp(method,'ifft'))
    X=abs(ifftshift(ifft2(fftshift(N))));
end
imagesc(X);
colormap(gray);
axis equal;
end

function [M,s,offset,time,fine,cv,method]=parseInput(varargin)
% PARSEINPUT parses the input for sheppLogan2D
% Author: Johannes Churt

%Initalization values
M=[];
s=256;
offset=[];
time=1;
fine=false;
cv=@computeValueNone;
setsize=false;
settime=false;
method='fft';

for k=1:nargin
    if (islogical(varargin{k}))
        %Logicals are for the level of detail
        fine=varargin{k};
    elseif (ischar(varargin{k}))
        %String reprasents the method of transforming to image-space
        switch varargin{k}
            case 'fft'
                method='fft';
            case 'ifft'
                method='ifft';
            otherwise
                error('SheppLogan:sheppLogan2D:invalidInputArgs',...
                   'Unknown transformation method %s.',varargin{k});
        end
    elseif (numel(varargin{k})==1)
        %First scalar is the size, second the time
        if (setsize && settime)
            error('SheppLogan:sheppLogan2D:invalidInputArgs',...
                'Invalid input arguments.');
        elseif (setsize)
            time=varargin{k};
            settime=true;
        else
            s=varargin{k};
            setsize=true;
        end
    elseif (ndims(varargin{k})==2 && size(varargin{k},2)==7)
        %User specified phantom
        M=varargin{k};
    elseif (ndims(varargin{k})==2 && size(varargin{k},2)==3)
        %An offset matrix moving the whole phantom
        offset=varargin{k};
        cv=@computeValueAll;
    elseif (ndims(varargin{k})==3 && size(varargin{k},2)==3)
        %An offset matrix moving components speratly
        offset=varargin{k};
        cv=@computeValueSpecific;
    else
        error('SheppLogan:sheppLogan2D:invalidInputArgs',...
            'Invalid input arguments.');
    end
end

if (isempty(M))
    %No phantom specified
    M=sheppLogan;
end

if (~isempty(offset))
    %We have a offset matrix, so scale it
    offset=[offset(:,1:2,:)/s*2 offset(:,3,:)*pi/180];
    if (time>size(offset,1))
    string=['th' 'st' 'nd' 'rd' repmat('th',1,6)];
    error('SheppLogan:sheppLogan2D:invalidInputArgs',...
        ['Offset matrix has only %d entries in time-direction, but '...
        '%d%s point of time was requested.'],...
        size(offset,1),time,string(2*rem(time,10)+1:2*rem(time,10)+2));
    end
end

numberellipses=size(M,1)-sum(M(:,7))*(fine==false);
if ((ndims(offset)==3) && (size(offset,3)~=numberellipses))
    error('SheppLogan:sheppLogan2D:invalidInputArgs',...
        'Offset matrix must have %d entries in z-direction.',numberellipses);
end
end
